@@ -27,7 +27,11 @@
 #define GLX_GLXEXT_PROTOTYPES
 #include <GL/glx.h>
 #include <GL/glxext.h>
+#include <vector>
 
+/**
+ * Canvas for rendering to an X11 window using GLX.
+ */
 class CanvasX11GLX : public CanvasX11
 {
 public:
@@ -38,13 +42,18 @@ public:
 protected:
     XVisualInfo *get_xvisualinfo();
     bool make_current();
+    bool reset_context();
     void swap_buffers() { glXSwapBuffers(xdpy_, xwin_); }
+    void get_glvisualconfig(GLVisualConfig &visual_config);
 
 private:
     bool check_glx_version();
     void init_extensions();
     bool ensure_glx_fbconfig();
     bool ensure_glx_context();
+    void init_gl_extensions();
+    void get_glvisualconfig_glx(GLXFBConfig config, GLVisualConfig &visual_config);
+    GLXFBConfig select_best_config(std::vector<GLXFBConfig> configs);
 
     GLXFBConfig glx_fbconfig_;
     GLXContext glx_context_;
