@@ -21,13 +21,14 @@
  *  Ben Smith (original glmark benchmark)
  *  Alexandros Frantzis (glmark2)
  */
+#include "mesh.h"
 #include "model.h"
 #include "vec.h"
 #include "log.h"
 #include "options.h"
 #include "util.h"
 #include "float.h"
-
+#include "math.h"
 #include <fstream>
 #include <sstream>
 #include <memory>
@@ -35,7 +36,6 @@
 using std::string;
 using std::vector;
 using LibMatrix::vec3;
-using LibMatrix::vec2;
 using LibMatrix::uvec3;
 
 #define read_or_fail(file, dst, size) do { \
@@ -48,6 +48,11 @@ using LibMatrix::uvec3;
     } \
 } while(0);
 
+/**
+ * Computes the bounding box for a Model::Object.
+ *
+ * @param object the Model object
+ */
 void
 Model::compute_bounding_box(const Object& object)
 {
@@ -89,9 +94,19 @@ Model::compute_bounding_box(const Object& object)
     minVec_ = vec3(minX, minY, minZ);
 }
 
+/**
+ * Appends the vertices of a Model::Object to a Mesh.
+ *
+ * @param object the object to append
+ * @param mesh the mesh to append to
+ * @param p_pos the attribute position to use for the 'position' attribute
+ * @param n_pos the attribute position to use for the 'normal' attribute
+ * @param t_pos the attribute position to use for the 'texcoord' attribute
+ */
 void
 Model::append_object_to_mesh(const Object &object, Mesh &mesh,
-                             int p_pos, int n_pos, int t_pos)
+                             int p_pos, int n_pos, int t_pos,
+                             int nt_pos, int nb_pos)
 {
     size_t face_count = object.faces.size();
 
@@ -109,6 +124,10 @@ Model::append_object_to_mesh(const Object &object, Mesh &mesh,
             mesh.set_attrib(n_pos, a.n);
         if (t_pos >= 0)
             mesh.set_attrib(t_pos, a.t);
+        if (nt_pos >= 0)
+            mesh.set_attrib(nt_pos, a.nt);
+        if (nb_pos >= 0)
+            mesh.set_attrib(nb_pos, a.nb);
 
         mesh.next_vertex();
         if (p_pos >= 0)
@@ -117,6 +136,10 @@ Model::append_object_to_mesh(const Object &object, Mesh &mesh,
             mesh.set_attrib(n_pos, b.n);
         if (t_pos >= 0)
             mesh.set_attrib(t_pos, b.t);
+        if (nt_pos >= 0)
+            mesh.set_attrib(nt_pos, b.nt);
+        if (nb_pos >= 0)
+            mesh.set_attrib(nb_pos, b.nb);
 
         mesh.next_vertex();
         if (p_pos >= 0)
@@ -125,10 +148,20 @@ Model::append_object_to_mesh(const Object &object, Mesh &mesh,
             mesh.set_attrib(n_pos, c.n);
         if (t_pos >= 0)
             mesh.set_attrib(t_pos, c.t);
+        if (nt_pos >= 0)
+            mesh.set_attrib(nt_pos, c.nt);
+        if (nb_pos >= 0)
+            mesh.set_attrib(nb_pos, c.nb);
     }
-
 }
 
+/**
+ * Converts a model to a mesh using the default attributes bindings.
+ *
+ * The default attributes and their order is: Position, Normal, Texcoord
+ *
+ * @param mesh the mesh to populate
+ */
 void
 Model::convert_to_mesh(Mesh &mesh)
 {
@@ -141,6 +174,14 @@ Model::convert_to_mesh(Mesh &mesh)
     convert_to_mesh(mesh, attribs);
 }
 
+/**
+ * Converts a model to a mesh using custom attribute bindings.
+ *
+ * The attribute bindings are pairs of <AttribType, dimensionality>.
+ *
+ * @param mesh the mesh to populate
+ * @param attribs the attribute bindings to use
+ */
 void
 Model::convert_to_mesh(Mesh &mesh,
                        const std::vector<std::pair<AttribType, int> > &attribs)
@@ -149,6 +190,8 @@ Model::convert_to_mesh(Mesh &mesh,
     int p_pos = -1;
     int n_pos = -1;
     int t_pos = -1;
+    int nt_pos = -1;
+    int nb_pos = -1;
 
     mesh.reset();
 
@@ -163,6 +206,10 @@ Model::convert_to_mesh(Mesh &mesh,
             n_pos = ai - attribs.begin();
         else if (ai->first == AttribTypeTexcoord)
             t_pos = ai - attribs.begin();
+        else if (ai->first == AttribTypeTangent)
+            nt_pos = ai - attribs.begin();
+        else if (ai->first == AttribTypeBitangent)
+            nb_pos = ai - attribs.begin();
     }
 
     mesh.set_vertex_format(format);
@@ -171,10 +218,41 @@ Model::convert_to_mesh(Mesh &mesh,
          iter != objects_.end();
          iter++)
     {
-        append_object_to_mesh(*iter, mesh, p_pos, n_pos, t_pos);
+        append_object_to_mesh(*iter, mesh, p_pos, n_pos, t_pos, nt_pos, nb_pos);
     }
 }
 
+void
+Model::calculate_texcoords()
+{
+    // Since the model didn't come with texcoords, and we don't actually know
+    // if it came with normals, either, we'll use positional spherical mapping
+    // to generate texcoords for the model.  See:
+    // http://www.mvps.org/directx/articles/spheremap.htm for more details.
+    vec3 centerVec = maxVec_ + minVec_;
+    centerVec *= 0.5;
+    
+    for (std::vector<Object>::iterator iter = objects_.begin();
+         iter != objects_.end();
+         iter++)
+    {
+        Object &object = *iter;
+        for (vector<Vertex>::iterator vertexIt = object.vertices.begin();
+             vertexIt != object.vertices.end();
+             vertexIt++)
+        {
+            Vertex& curVertex = *vertexIt;
+            vec3 vnorm(curVertex.v - centerVec);
+            vnorm.normalize();
+            curVertex.t.x(asinf(vnorm.x()) / M_PI + 0.5);
+            curVertex.t.y(asinf(vnorm.y()) / M_PI + 0.5);
+        }
+    }
+}
+
+/**
+ * Calculates the normal vectors of the model vertices.
+ */
 void
 Model::calculate_normals()
 {
@@ -185,28 +263,72 @@ Model::calculate_normals()
          iter++)
     {
         Object &object = *iter;
-        size_t face_count = object.faces.size();
-        size_t vertex_count = object.vertices.size();
 
-        for(unsigned i = 0; i < face_count; i++)
+        for (vector<Face>::const_iterator f_iter = object.faces.begin();
+             f_iter != object.faces.end();
+             f_iter++)
         {
-            const Face &face = object.faces[i];
+            const Face &face = *f_iter;
             Vertex &a = object.vertices[face.a];
             Vertex &b = object.vertices[face.b];
             Vertex &c = object.vertices[face.c];
 
+            /* Calculate normal */
             n = LibMatrix::vec3::cross(b.v - a.v, c.v - a.v);
             n.normalize();
             a.n += n;
             b.n += n;
             c.n += n;
+
+            LibMatrix::vec3 q1(b.v - a.v);
+            LibMatrix::vec3 q2(c.v - a.v);
+            LibMatrix::vec2 u1(b.t - a.t);
+            LibMatrix::vec2 u2(c.t - a.t);
+            float det = (u1.x() * u2.y() - u2.x() * u1.y());
+
+            /* Calculate tangent */
+            LibMatrix::vec3 nt;
+            nt.x(det * (u2.y() * q1.x() - u1.y() * q2.x()));
+            nt.y(det * (u2.y() * q1.y() - u1.y() * q2.y()));
+            nt.z(det * (u2.y() * q1.z() - u1.y() * q2.z()));
+            nt.normalize();
+            a.nt += nt;
+            b.nt += nt;
+            c.nt += nt;
+
+            /* Calculate bitangent */
+            LibMatrix::vec3 nb;
+            nb.x(det * (u1.x() * q2.x() - u2.x() * q1.x()));
+            nb.y(det * (u1.x() * q2.y() - u2.x() * q1.y()));
+            nb.z(det * (u1.x() * q2.z() - u2.x() * q1.z()));
+            nb.normalize();
+            a.nb += nb;
+            b.nb += nb;
+            c.nb += nb;
+        }
+
+        for (vector<Vertex>::iterator v_iter = object.vertices.begin();
+             v_iter != object.vertices.end();
+             v_iter++)
+        {
+            Vertex &v = *v_iter;
+            /* Orthogonalize */
+            v.nt = (v.nt - v.n * LibMatrix::vec3::dot(v.nt, v.n));
+            v.n.normalize();
+            v.nt.normalize();
+            v.nb.normalize();
         }
 
-        for(unsigned i = 0; i < vertex_count; i++)
-            object.vertices[i].n.normalize();
     }
 }
 
+/**
+ * Load a model from a 3DS file.
+ *
+ * @param filename the name of the file
+ *
+ * @return whether loading succeeded
+ */
 bool
 Model::load_3ds(const std::string &filename)
 {
@@ -349,6 +471,7 @@ Model::load_3ds(const std::string &filename)
                     object->vertices[i].t.y(f[1]);
                 }
                 }
+                gotTexcoords_ = true;
                 break;
 
             //----------- Skip unknow chunks ------------
@@ -377,17 +500,14 @@ Model::load_3ds(const std::string &filename)
     return true;
 }
 
-template<typename T> T
-fromString(const string& asString)
-{
-    std::stringstream ss(asString);
-    T retVal;
-    ss >> retVal;
-    return retVal;
-}
-
-void
-get_values(const string& source, vec3& v)
+/**
+ * Parse vec3 values from an OBJ file.
+ *
+ * @param source the source line to parse
+ * @param v the vec3 to populate
+ */
+static void
+obj_get_values(const string& source, vec3& v)
 {
     // Skip the definition type...
     string::size_type endPos = source.find(" ");
@@ -407,7 +527,7 @@ get_values(const string& source, vec3& v)
     }
     string::size_type numChars(endPos - startPos);
     string xs(source, startPos, numChars);
-    float x = fromString<float>(xs);
+    float x = Util::fromString<float>(xs);
     // Then the second value...
     startPos = endPos + 1;
     endPos = source.find(" ", startPos);
@@ -418,7 +538,7 @@ get_values(const string& source, vec3& v)
     }
     numChars = endPos - startPos;
     string ys(source, startPos, numChars);
-    float y = fromString<float>(ys);
+    float y = Util::fromString<float>(ys);
     // And the third value (there might be a fourth, but we don't care)...
     startPos = endPos + 1;
     endPos = source.find(" ", startPos);
@@ -431,14 +551,20 @@ get_values(const string& source, vec3& v)
         numChars = endPos - startPos;
     }
     string zs(source, startPos, endPos - startPos);
-    float z = fromString<float>(zs);
+    float z = Util::fromString<float>(zs);
     v.x(x);
     v.y(y);
-    v.z(z);    
+    v.z(z);
 }
 
-void
-get_values(const string& source, vec2& v)
+/**
+ * Parse uvec3 values from an OBJ file.
+ *
+ * @param source the source line to parse
+ * @param v the uvec3 to populate
+ */
+static void
+obj_get_values(const string& source, uvec3& v)
 {
     // Skip the definition type...
     string::size_type endPos = source.find(" ");
@@ -458,46 +584,7 @@ get_values(const string& source, vec2& v)
     }
     string::size_type numChars(endPos - startPos);
     string xs(source, startPos, numChars);
-    float x = fromString<float>(xs);
-    // Then the second value (there might be a third, but we don't care)...
-    startPos = endPos + 1;
-    endPos = source.find(" ", startPos);
-    if (endPos == string::npos)
-    {
-        numChars = endPos;
-    }
-    else
-    {
-        numChars = endPos - startPos;
-    }
-    string ys(source, startPos, numChars);
-    float y = fromString<float>(ys);
-    v.x(x);
-    v.y(y);
-}
-
-void
-get_values(const string& source, uvec3& v)
-{
-    // Skip the definition type...
-    string::size_type endPos = source.find(" ");
-    string::size_type startPos(0);
-    if (endPos == string::npos)
-    {
-        Log::error("Bad element '%s'\n", source.c_str());
-        return;
-    }
-    // Find the first value...
-    startPos = endPos + 1;
-    endPos = source.find(" ", startPos);
-    if (endPos == string::npos)
-    {
-        Log::error("Bad element '%s'\n", source.c_str());
-        return;
-    }
-    string::size_type numChars(endPos - startPos);
-    string xs(source, startPos, numChars);
-    unsigned int x = fromString<unsigned int>(xs);
+    unsigned int x = Util::fromString<unsigned int>(xs);
     // Then the second value...
     startPos = endPos+1;
     endPos = source.find(" ", startPos);
@@ -508,7 +595,7 @@ get_values(const string& source, uvec3& v)
     }
     numChars = endPos - startPos;
     string ys(source, startPos, numChars);
-    unsigned int y = fromString<unsigned int>(ys);
+    unsigned int y = Util::fromString<unsigned int>(ys);
     // And the third value (there might be a fourth, but we don't care)...
     startPos = endPos + 1;
     endPos = source.find(" ", startPos);
@@ -521,16 +608,26 @@ get_values(const string& source, uvec3& v)
         numChars = endPos - startPos;
     }
     string zs(source, startPos, numChars);
-    unsigned int z = fromString<unsigned int>(zs);
+    unsigned int z = Util::fromString<unsigned int>(zs);
     v.x(x);
     v.y(y);
-    v.z(z);    
+    v.z(z);
 }
 
+/**
+ * Load a model from an OBJ file.
+ *
+ * @param filename the name of the file
+ *
+ * @return whether loading succeeded
+ */
 bool
 Model::load_obj(const std::string &filename)
 {
-    std::ifstream inputFile(filename.c_str());
+    Log::debug("Loading model from obj file '%s'\n", filename.c_str());
+
+    const std::auto_ptr<std::istream> input_file_ptr(Util::get_resource(filename));
+    std::istream& inputFile(*input_file_ptr);
     if (!inputFile)
     {
         Log::error("Failed to open '%s'\n", filename.c_str());
@@ -566,7 +663,7 @@ Model::load_obj(const std::string &filename)
         if (definitionType == vertex_definition)
         {
             Vertex v;
-            get_values(curSrc, v.v);
+            obj_get_values(curSrc, v.v);
             object.vertices.push_back(v);
         }
         else if (definitionType == normal_definition)
@@ -584,7 +681,7 @@ Model::load_obj(const std::string &filename)
         else if (definitionType == face_definition)
         {
             uvec3 v;
-            get_values(curSrc, v);
+            obj_get_values(curSrc, v);
             Face f;
             // OBJ models index from '1'.
             f.a = v.x() - 1;
@@ -600,3 +697,111 @@ Model::load_obj(const std::string &filename)
         object.vertices.size(), object.faces.size());
     return true;
 }
+
+namespace ModelPrivate
+{
+ModelMap modelMap;
+}
+
+/**
+ * Locate all available models.
+ *
+ * This method scans the built-in data paths and build a database of usable
+ * models available to scenes.  Map is available on a read-only basis to scenes
+ * that might find it useful for listing models, etc.
+ *
+ * @return a map containing information about the located models
+ */
+const ModelMap&
+Model::find_models()
+{
+    if (!ModelPrivate::modelMap.empty())
+    {
+        return ModelPrivate::modelMap;
+    }
+    vector<string> pathVec;
+    string dataDir(GLMARK_DATA_PATH"/models");
+    Util::list_files(dataDir, pathVec);
+#ifdef GLMARK_EXTRAS_PATH
+    string extrasDir(GLMARK_EXTRAS_PATH"/models");
+    Util::list_files(extrasDir, pathVec);
+#endif
+
+    // Now that we have a list of all of the model files available to us,
+    // let's go through and pull out the names and what format they're in
+    // so the scene can decide which ones to use.
+    for(vector<string>::const_iterator pathIt = pathVec.begin();
+        pathIt != pathVec.end();
+        pathIt++)
+    {
+        const string& curPath = *pathIt;
+        string::size_type namePos(0);
+        string::size_type slashPos = curPath.rfind("/");
+        if (slashPos != string::npos)
+        {
+            // Advance to the first character after the last slash
+            namePos = slashPos + 1;
+        }
+
+        ModelFormat format(MODEL_INVALID);
+        string::size_type extPos = curPath.rfind(".3ds");
+        if (extPos == string::npos)
+        {
+            // It's not a 3ds model
+            extPos = curPath.rfind(".obj");
+            if (extPos == string::npos)
+            {
+                // It's not an obj model either, so skip it.
+                continue;
+            }
+            format = MODEL_OBJ;
+        }
+        else
+        {
+            // It's a 3ds model
+            format = MODEL_3DS;
+        }
+
+        string name(curPath, namePos, extPos - namePos);
+        ModelDescriptor* desc = new ModelDescriptor(name, format, curPath);
+        ModelPrivate::modelMap.insert(std::make_pair(name, desc));
+    }
+
+    return ModelPrivate::modelMap;
+}
+
+/**
+ * Load a model by name.
+ *
+ * You must initialize the available model collection using
+ * Model::find_models() before using this method.
+ *
+ * @param modelName the model name
+ *
+ * @return whether the operation succeeded
+ */
+bool
+Model::load(const string& modelName)
+{
+    bool retVal(false);
+    ModelMap::const_iterator modelIt = ModelPrivate::modelMap.find(modelName);
+    if (modelIt == ModelPrivate::modelMap.end())
+    {
+        return retVal;
+    }
+
+    ModelDescriptor* desc = modelIt->second;
+    switch (desc->format())
+    {
+        case MODEL_INVALID:
+            break;
+        case MODEL_3DS:
+            retVal = load_3ds(desc->pathname());
+            break;
+        case MODEL_OBJ:
+            retVal = load_obj(desc->pathname());
+            break;
+    }
+
+    return retVal;
+}
